<?php
/**
 * @package まいるど文庫 for WordPress
 */

// デフォルト設定値
define( 'MAIRUDO_BUNKO_OPSIONS', serialize( array(
  'create_account' => 1,
  'wrap_image_size' => 1,
  'login_logo_image_id' => ''
) ) );

// 画像サイズ
define( 'MAIRUDO_BUNKO_IMAGE_SIZE_THUMBNAIL', 48 );
define( 'MAIRUDO_BUNKO_IMAGE_SIZE_MEDIUM', 96 );
define( 'MAIRUDO_BUNKO_IMAGE_SIZE_LARGE', 168 );

// 省略文字数
define( 'MAIRUDO_BUNKO_EXCERPT_LENGTH', 100 );

require_once get_template_directory(). '/inc/bunko.php';
require_once get_template_directory(). '/lib/theme-update-checker.php';
new ThemeUpdateChecker(
  'mairudo-bunko-for-wordpress',
  'http://maroyaka.coln.biz/update/wp/themes/mairudo-bunko-for-wordpress.json'
);

///////////////////////////////////////
/* 画像表示 */
///////////////////////////////////////
function the_post_thumbnail_wrap( $size, $attr ) {

	switch ( $size ) {
		case 'thumbnail':
			$size = array( MAIRUDO_BUNKO_IMAGE_SIZE_THUMBNAIL, MAIRUDO_BUNKO_IMAGE_SIZE_THUMBNAIL );
			break;
		case 'medium':
			$size = array( MAIRUDO_BUNKO_IMAGE_SIZE_MEDIUM, MAIRUDO_BUNKO_IMAGE_SIZE_MEDIUM );
			break;
		case 'large':
			$size = array( MAIRUDO_BUNKO_IMAGE_SIZE_LARGE, MAIRUDO_BUNKO_IMAGE_SIZE_LARGE );
			break;
	}

	if ( has_post_thumbnail() ) {
		the_post_thumbnail( $size, $attr );
	} else {
		$html = '';
		foreach ( $attr as $name => $value ) {
			$html .= " $name=". '"'. esc_attr($value). '"';
		}
		// デフォルト
		if ( is_array( $size ) ) {
			list($w, $h) = $size;
			echo '<img src="'. get_template_directory_uri(). '/img/full_noimage_novel.png" width="'. $w.'" height="'. $h.'"'. $html. ' />';
		} else {
			echo '<img src="'. get_template_directory_uri(). '/img/'. $size. '_noimage_novel.png"'. $html. ' />';
		}
	}
}

///////////////////////////////////////
/* ページネーション */
///////////////////////////////////////
function pagination( $pages = '', $range = 2 ) {
	global $paged, $wp_query;

	$showitems = ($range * 2) + 1;

	if (empty($paged)) $paged = 1;

	if ( $pages == '' ) {
		$pages = $wp_query->max_num_pages;
		if ( !$pages ) {
			$pages = 1;
		}
	}

	if ( 1 != $pages ) {
		echo '<ul class="pagination pagination-sm">';

		for ( $i = 1; $i <= $pages; $i++ ) {
			if ( 1 != $pages && ( !($i >= $paged+$range+1 || $i <= $paged-$range-1) || $pages <= $showitems ) ) {
				echo ($paged == $i) ? '<li class="active"><a>'. $i .'</a></li>' : '<li><a href="'. get_pagenum_link($i). '">'. $i .'</a></li>';
			}
		}
		echo '</ul><br>';

		echo '<ul class="pagination pagination-sm">';

		echo '<li class="disabled"><a>'. $paged. ' / '. $wp_query->max_num_pages. '</a></li>';

		if ( $paged > 1 ) {
			echo '<li class="prev"><a href="'. get_pagenum_link( $paged - 1 ). '">前のページ</a></li>';
		} else {
			echo '<li class="disabled"><a>前のページ</a></li>';
		}
		if ($paged < $pages) {
			echo '<li class="next"><a href="'. get_pagenum_link( $paged + 1 ). '">次のページ</a></li>';
		} else {
			echo '<li class="disabled"><a>次のページ</a></li>';
		}
		echo '</ul>';

	}
}

///////////////////////////////////////
/* 小説コメント */
///////////////////////////////////////
function novel_comments( $comment, $args, $depth ) {
	global $post;
	$GLOBALS['comment'] = $comment;

	?>
	<li <?php comment_class( 'media comment' ); ?> id="comment-<?php comment_ID(); ?>">
		<?php if ( wp_is_mobile() ) : ?>
			<div class="comment-meta clearfix">
				<?php echo get_avatar( $comment, 48, null, null, array( 'class' => 'comment-image img-thumbnail' ) ); ?>
				<strong><?php comment_author(); ?></strong> |
				<?php comment_date(); ?>
				<?php comment_time(); ?> |
				<?php comment_reply_link( array_merge( $args, array( 'depth' => $depth, 'max_depth' => $args['max_depth'] ) ) ); ?>
			</div>
			<div class="comment-text">
				<?php if ( $comment->comment_approved == '0' ) : ?>
					<div class="text-danger">このコメントは承認待ちです。</div>
				<?php endif; ?>
				<?php comment_text(); ?>
			</div>
		<?php else : ?>
			<div class="media-left">
				<?php echo get_avatar( $comment, 48, null, null, array( 'class' => 'media-object img-thumbnail' ) ); ?>
			</div>
			<div class="media-body">
				<div class="comment-meta">
					<strong><?php comment_author(); ?></strong> |
					<?php comment_date(); ?>
					<?php comment_time(); ?> |
					<?php comment_reply_link( array_merge( $args, array( 'depth' => $depth, 'max_depth' => $args['max_depth'] ) ) ); ?>
				</div>
				<div class="comment-text">
					<?php if ( $comment->comment_approved == '0' ) : ?>
						<div class="text-danger">このコメントは承認待ちです。</div>
					<?php endif; ?>
					<?php comment_text(); ?>
				</div>
			</div>
		<?php endif;
}

///////////////////////////////////////
/* 活動日記コメント */
///////////////////////////////////////
function diary_comments( $comment, $args, $depth ) {
	global $post;
	$GLOBALS['comment'] = $comment;

	?>
	<li <?php comment_class( 'media comment' ); ?> id="comment-<?php comment_ID(); ?>">
		<div class="media-left">
			<img src="http://localhost/wordpress/wp-content/themes/bunko/image/medium_noimage_user.png" class="media-object img-thumbnail">
		</div>
		<div class="media-body">
			<div class="comment-meta">
				<strong><?php comment_author(); ?></strong> |
				<?php comment_date(); ?>
				<?php comment_time(); ?> |
				<?php comment_reply_link( array_merge( $args, array( 'depth' => $depth, 'max_depth' => $args['max_depth'] ) ) ); ?>
			</div>
			<div class="comment-text">
				<?php if ( $comment->comment_approved == '0' ) : ?>
					<div class="text-danger">このコメントは承認待ちです。</div>
				<?php endif; ?>
				<?php comment_text(); ?>
			</div>
		</div>
	<?php
}

///////////////////////////////////////
/* カレンダー */
///////////////////////////////////////
function ucc_get_calendar( $post_types = '' , $initial = true , $echo = true ) {
  global $wpdb, $m, $monthnum, $year, $wp_locale, $post;

  if ( is_single() ) {
    $author = $post->post_author;
  } elseif ( is_date() || is_month() || is_year() ) {
    $author = ( isset( $_GET['uid'] ) ) ? $_GET['uid'] : null;
  }
  if ( empty($author) ) return;

  if ( empty( $post_types ) || !is_array( $post_types ) ) {
    $args = array(
      'public' => true ,
      '_builtin' => false
    );
    $output = 'names';
    $operator = 'and';

    $post_types = get_post_types( $args , $output , $operator );
    $post_types = array_merge( $post_types , array( 'post' ) );
  } else {
    /* Trust but verify. */
    $my_post_types = array();
    foreach ( $post_types as $post_type ) {
      if ( post_type_exists( $post_type ) )
        $my_post_types[] = $post_type;
    }
    $post_types = $my_post_types;
  }
  $post_types_key = implode( '' , $post_types );
  $post_types = "'" . implode( "' , '" , $post_types ) . "'";

  $cache = array();
  $key = md5( $m . $monthnum . $year . $post_types_key );
  if ( $cache = wp_cache_get( 'get_calendar' , 'calendar' ) ) {
    if ( is_array( $cache ) && isset( $cache[$key] ) ) {
      remove_filter( 'get_calendar' , 'ucc_get_calendar_filter' );
      $output = apply_filters( 'get_calendar',  $cache[$key] );
      add_filter( 'get_calendar' , 'ucc_get_calendar_filter' );
      if ( $echo ) {
        echo $output;
        return;
      } else {
        return $output;
      }
    }
  }

  if ( !is_array( $cache ) )
    $cache = array();

  // Quick check. If we have no posts at all, abort!
  if ( empty( $posts ) ) {
    $sql = "SELECT 1 as test FROM $wpdb->posts WHERE post_author = '$author' AND post_type IN ( $post_types ) AND post_status = 'publish' LIMIT 1";
    $gotsome = $wpdb->get_var( $sql );
    if ( !$gotsome ) {
      $cache[$key] = '';
      wp_cache_set( 'get_calendar' , $cache , 'calendar' );
      return;
    }
  }

  if ( isset( $_GET['w'] ) )
    $w = '' . intval( $_GET['w'] );

  // week_begins = 0 stands for Sunday
  $week_begins = intval( get_option( 'start_of_week' ) );

  // Let's figure out when we are
  if ( !empty( $monthnum ) && !empty( $year ) ) {
    $thismonth = '' . zeroise( intval( $monthnum ) , 2 );
    $thisyear = ''.intval($year);
  } elseif ( !empty( $w ) ) {
    // We need to get the month from MySQL
    $thisyear = '' . intval( substr( $m , 0 , 4 ) );
    $d = ( ( $w - 1 ) * 7 ) + 6; //it seems MySQL's weeks disagree with PHP's
    $thismonth = $wpdb->get_var( "SELECT DATE_FORMAT( ( DATE_ADD( '${thisyear}0101' , INTERVAL $d DAY ) ) , '%m' ) " );
  } elseif ( !empty( $m ) ) {
    $thisyear = '' . intval( substr( $m , 0 , 4 ) );
    if ( strlen( $m ) < 6 )
        $thismonth = '01';
    else
        $thismonth = '' . zeroise( intval( substr( $m , 4 , 2 ) ) , 2 );
  } else {
    $thisyear = gmdate( 'Y' , current_time( 'timestamp' ) );
    $thismonth = gmdate( 'm' , current_time( 'timestamp' ) );
  }

  $unixmonth = mktime( 0 , 0 , 0 , $thismonth , 1 , $thisyear);

  // Get the next and previous month and year with at least one post
  $previous = $wpdb->get_row( "SELECT DISTINCT MONTH( post_date ) AS month , YEAR( post_date ) AS year
    FROM $wpdb->posts
    WHERE post_date < '$thisyear-$thismonth-01'
    AND post_author = '$author'
    AND post_type IN ( $post_types ) AND post_status = 'publish'
      ORDER BY post_date DESC
      LIMIT 1" );
  $next = $wpdb->get_row( "SELECT DISTINCT MONTH( post_date ) AS month, YEAR( post_date ) AS year
    FROM $wpdb->posts
    WHERE post_date > '$thisyear-$thismonth-01'
    AND MONTH( post_date ) != MONTH( '$thisyear-$thismonth-01' )
    AND post_author = '$author'
    AND post_type IN ( $post_types ) AND post_status = 'publish'
      ORDER  BY post_date ASC
      LIMIT 1" );

  $weeks = array(
   'start-of-week-sun',
   'start-of-week-mon',
   'start-of-week-tue',
   'start-of-week-wed',
   'start-of-week-thurs',
   'start-of-week-fri',
   'start-of-week-sat',
  );
  /* translators: Calendar caption: 1: month name, 2: 4-digit year */
  $calendar_caption = _x( '%1$s %2$s' , 'calendar caption' );
  $calendar_output = '<table id="wp-calendar" summary="' . esc_attr__( 'Calendar' ) . '" class="table table-bordered '. $weeks[$week_begins]. '">
  <caption>' . sprintf( $calendar_caption , $wp_locale->get_month( $thismonth ) , date( 'Y' , $unixmonth ) ) . '</caption>
  <thead>
  <tr>';

  $myweek = array();

  for ( $wdcount = 0 ; $wdcount <= 6 ; $wdcount++ ) {
    $myweek[] = $wp_locale->get_weekday( ( $wdcount + $week_begins ) % 7 );
  }

  foreach ( $myweek as $wd ) {
    $day_name = ( true == $initial ) ? $wp_locale->get_weekday_initial( $wd ) : $wp_locale->get_weekday_abbrev( $wd );
    $wd = esc_attr( $wd );
    $calendar_output .= "\n\t\t<th scope=\"col\" title=\"$wd\">$day_name</th>";
  }

  $calendar_output .= '
  </tr>
  </thead>

  <tfoot>
  <tr>';

  if ( $previous ) {    $calendar_output .= "\n\t\t" . '<td colspan="3" id="prev"><a href="' . EH_get_post_type_date_link( $post_types_key, $previous->year , $previous->month ) . '?uid='. $author. '" title="' . sprintf( __( 'View posts for %1$s %2$s' ) , $wp_locale->get_month( $previous->month ) , date( 'Y' , mktime( 0 , 0 , 0 , $previous->month , 1 , $previous->year ) ) ) . '">&laquo; ' . $wp_locale->get_month_abbrev( $wp_locale->get_month( $previous->month ) ) . '</a></td>';
  } else {
    $calendar_output .= "\n\t\t" . '<td colspan="3" id="prev" class="pad">&nbsp;</td>';
  }

  $calendar_output .= "\n\t\t" . '<td class="pad">&nbsp;</td>';

  if ( $next ) {    $calendar_output .= "\n\t\t" . '<td colspan="3" id="next"><a href="' . EH_get_post_type_date_link( $post_types_key, $next->year , $next->month ) . '?uid='. $author. '" title="' . esc_attr( sprintf( __( 'View posts for %1$s %2$s' ) , $wp_locale->get_month( $next->month ) , date( 'Y' , mktime( 0 , 0 , 0 , $next->month , 1 , $next->year ) ) ) ) . '">' . $wp_locale->get_month_abbrev( $wp_locale->get_month( $next->month ) ) . ' &raquo;</a></td>';
  } else {
    $calendar_output .= "\n\t\t" . '<td colspan="3" id="next" class="pad">&nbsp;</td>';
  }

  $calendar_output .= '
  </tr>
  </tfoot>

  <tbody>
  <tr>';

  // Get days with posts
  $dayswithposts = $wpdb->get_results( "SELECT DISTINCT DAYOFMONTH( post_date )
    FROM $wpdb->posts WHERE MONTH( post_date ) = '$thismonth'
    AND YEAR( post_date ) = '$thisyear'
    AND post_author = '$author'
    AND post_type IN ( $post_types ) AND post_status = 'publish'
    AND post_date < '" . current_time( 'mysql' ) . '\'', ARRAY_N );
  if ( $dayswithposts ) {
    foreach ( (array) $dayswithposts as $daywith ) {
      $daywithpost[] = $daywith[0];
    }
  } else {
    $daywithpost = array();
  }

  if ( strpos( $_SERVER['HTTP_USER_AGENT'] , 'MSIE' ) !== false || stripos( $_SERVER['HTTP_USER_AGENT'] , 'camino' ) !== false || stripos( $_SERVER['HTTP_USER_AGENT'] , 'safari' ) !== false )
    $ak_title_separator = "\n";
  else
    $ak_title_separator = ', ';

  $ak_titles_for_day = array();
  $ak_post_titles = $wpdb->get_results( "SELECT ID, post_title, DAYOFMONTH( post_date ) as dom "
    . "FROM $wpdb->posts "
    . "WHERE YEAR( post_date ) = '$thisyear' "
    . "AND MONTH( post_date ) = '$thismonth' "
    . "AND post_date < '" . current_time( 'mysql' ) . "' "
    . "AND post_author = '$author'"
    . "AND post_type IN ( $post_types ) AND post_status = 'publish'"
  );
  if ( $ak_post_titles ) {
    foreach ( (array) $ak_post_titles as $ak_post_title ) {

        $post_title = esc_attr( apply_filters( 'the_title' , $ak_post_title->post_title , $ak_post_title->ID ) );

        if ( empty( $ak_titles_for_day['day_' . $ak_post_title->dom] ) )
          $ak_titles_for_day['day_'.$ak_post_title->dom] = '';
        if ( empty( $ak_titles_for_day["$ak_post_title->dom"] ) ) // first one
          $ak_titles_for_day["$ak_post_title->dom"] = $post_title;
        else
          $ak_titles_for_day["$ak_post_title->dom"] .= $ak_title_separator . $post_title;
    }
  }

  // See how much we should pad in the beginning
  $pad = calendar_week_mod( date( 'w' , $unixmonth ) - $week_begins );
  if ( 0 != $pad )
    $calendar_output .= "\n\t\t" . '<td colspan="' . esc_attr( $pad ) . '" class="pad">&nbsp;</td>';

  $daysinmonth = intval( date( 't' , $unixmonth ) );
  for ( $day = 1 ; $day <= $daysinmonth ; ++$day ) {
    if ( isset( $newrow ) && $newrow )
      $calendar_output .= "\n\t</tr>\n\t<tr>\n\t\t";
    $newrow = false;

    if ( $day == gmdate( 'j' , current_time( 'timestamp' ) ) && $thismonth == gmdate( 'm' , current_time( 'timestamp' ) ) && $thisyear == gmdate( 'Y' , current_time( 'timestamp' ) ) )
      $calendar_output .= '<td id="today">';
    else
      $calendar_output .= '<td>';

    if ( in_array( $day , $daywithpost ) ) // any posts today?
        $calendar_output .= '<a href="' . EH_get_post_type_date_link( $post_types_key, $thisyear , $thismonth , $day ) . "?uid=$author\" title=\"" . esc_attr( $ak_titles_for_day[$day] ) . "\">$day</a>";
    else
      $calendar_output .= $day;
    $calendar_output .= '</td>';

    if ( 6 == calendar_week_mod( date( 'w' , mktime( 0 , 0 , 0 , $thismonth , $day , $thisyear ) ) - $week_begins ) )
      $newrow = true;
  }

  $pad = 7 - calendar_week_mod( date( 'w' , mktime( 0 , 0 , 0 , $thismonth , $day , $thisyear ) ) - $week_begins );
  if ( $pad != 0 && $pad != 7 )
    $calendar_output .= "\n\t\t" . '<td class="pad" colspan="' . esc_attr( $pad ) . '">&nbsp;</td>';

  $calendar_output .= "\n\t</tr>\n\t</tbody>\n\t</table>";

  $cache[$key] = $calendar_output;
  wp_cache_set( 'get_calendar' , $cache, 'calendar' );

  remove_filter( 'get_calendar' , 'ucc_get_calendar_filter' );
  $output = apply_filters( 'get_calendar',  $calendar_output );
  add_filter( 'get_calendar' , 'ucc_get_calendar_filter' );

  if ( $echo )
    echo $output;
  else
    return $output;
}

function ucc_get_calendar_filter( $content ) {
  $output = ucc_get_calendar( '' , '' , false );
  return $output;
}
add_filter( 'get_calendar' , 'ucc_get_calendar_filter' , 10 , 2 );

///////////////////////////////////////
/* 日付指定リンク生成 */
///////////////////////////////////////
function EH_get_post_type_date_link( $post_type, $year, $month = 0, $day = 0 ) {
    global $wp_rewrite;
    $post_type_obj = get_post_type_object( $post_type );
    $post_type_slug = $post_type_obj->rewrite['slug'] ? $post_type_obj->rewrite['slug'] : $post_type_obj->name;
    if( $day ) { // day archive link
        // set to today's values if not provided
        if ( !$year )
            $year = gmdate('Y', current_time('timestamp'));
        if ( !$month )
            $month = gmdate('m', current_time('timestamp'));
        $link = $wp_rewrite->get_day_permastruct();
    } else if ( $month ) { // month archive link
        if ( !$year )
            $year = gmdate('Y', current_time('timestamp'));
        $link = $wp_rewrite->get_month_permastruct();
    } else { // year archive link
        $link = $wp_rewrite->get_year_permastruct();
    }
    if ( !empty($link) ) {
        $link = str_replace('%year%', $year, $link);
        $link = str_replace('%monthnum%', zeroise(intval($month), 2), $link );
        $link = str_replace('%day%', zeroise(intval($day), 2), $link );
        return home_url( "$post_type_slug$link" );
    }
    return home_url( $post_type_slug );
}

///////////////////////////////////////
/* カスタムフィールド配列指定 */
///////////////////////////////////////
function get_post_meta_arr( $post_id, $key, $single ) {
  $value = get_post_meta( $post_id, $key, $single );
  if ( empty($value) ) $value = array();
  return $value;
}
