<?php
new MairudoBunkoAction();

class MairudoBunkoAction {

	///////////////////////////////////////
	/* 初期定義 */
	///////////////////////////////////////
	public function __construct() {
		global $mairudo_bunko_option;

		// 処理分岐
		if (getenv('REQUEST_METHOD') === 'POST') {

			$post_actions = array( 'presignupRegist', 'signupRegist', 'dreamRegist' );
			$action = ( isset( $_POST['act'] ) ? $_POST['act'] : '' );

			if ( in_array( $action, $post_actions ) ) {
				if ( !check_admin_referer( $action, '_nonce' ) || $mairudo_bunko_option['create_account'] != 1 ) {
					$this->error( '不正なアクセスです' );
				}
				$this->{$action}();
			}

		} else {

			$get_actions = array( 'presignup', 'signup', 'favorites' );
			$action = ( isset( $_GET['act'] ) ? $_GET['act'] : '' );

			if ( in_array( $action, $get_actions ) ) {
				if ( method_exists( $this, $action ) ) $this->{$action}();
				add_filter( 'wp_title_parts', array( $this, "{$action}Title" ) );
			}

		}

	}

	///////////////////////////////////////
	/* 名前変換 */
	///////////////////////////////////////
	// POSTアクション
	public function dreamRegist() {
		$post_id = absint( $_POST['post_id'] );
		$current_id = absint( $_POST['current_id'] );

		$dreams = array();
		if ( isset( $_POST['dreams'] ) ) {
			foreach ( $_POST['dreams'] as $dream ) {
				$dreams[] = array(
					'name' => esc_attr( $dream['name'] )
				);
			}
		}
		$dreams = base64_encode( json_encode( $dreams ) );
		$cname = 'dream-' . $post_id;
		$expire = time() + 60*60*24*7;

		setcookie( $cname, $dreams, $expire );

		wp_safe_redirect( get_permalink( $current_id ) );
		exit;
	}

	///////////////////////////////////////
	/* ユーザーお気に入り小説 */
	///////////////////////////////////////
	// タイトル
	public function favoritesTitle( $title_array ) {
		$title_array[] = 'お気に入り小説';
		return $title_array;
	}
	// GETアクション
	public function favorites() {
		if ( !is_user_logged_in() ) {
			wp_safe_redirect( wp_login_url() );
			exit;
		}
	}

	///////////////////////////////////////
	/* 仮登録 */
	///////////////////////////////////////
	// タイトル
	public function presignupTitle( $title_array ) {
		$title_array[] = '新規登録';
		return $title_array;
	}
	// POSTアクション
	public function presignupRegist() {

		// バリデーション
		if ( !isset( $_POST['email'] ) || empty( $_POST['email'] ) ) {
			$this->error( 'メールアドレスを入力してください。' );
		}
		$user =  get_user_by( 'email', $_POST['email'] );
		if ( $user !== false ) $this->error( 'そのメールアドレスは既に使用されています' );

		// トークン取得
		$tokens = get_option( 'user_tokens', array() );

		// 古いトークンを破棄
		foreach ( $tokens as $idx => $token ) {
			if ( $token['timeout'] + (60*60*24) < time() || $token['email'] == $_POST['email'] ) {
				unset( $tokens[$idx] );
			}
		}

		// トークン生成
		$token = hash( 'sha256', time(). $_POST['email'] );

		// トークン追加
		$tokens[] = array(
			'email' => $_POST['email'],
			'token' => $token,
			'timeout' => time()
		);
		update_option( 'user_tokens', $tokens );

		// メール送信
		$subject = '登録を受け付けました';
		ob_start();
		require_once get_template_directory(). '/inc/email/activete.php';
		$body = ob_get_contents();
		ob_end_clean();
		wp_mail( $_POST['email'], $subject, $body );
	}

	///////////////////////////////////////
	/* 本登録 */
	///////////////////////////////////////
	// タイトル
	public function signupTitle( $title_array ) {
		array_unshift( $title_array, '新規登録' );
		return $title_array;
	}
	// GETアクション
	public function signup() {

		// バリデーション
		if ( !isset( $_GET['t'] ) ) $this->error( '不正なアクセスです' );

		// トークン取得
		$tokens = get_option( 'user_tokens', array() );

		// トークン検索
		$GLOBALS['user_email'] = '';
		foreach ( $tokens as $idx => $token ) {
			if ( $token['token'] === $_GET['t'] ) {
				$GLOBALS['user_email'] = $token['email'];
				unset($tokens[$idx]);
			}
		}

		$user =  get_user_by( 'email', $GLOBALS['user_email'] );
		if ( empty( $GLOBALS['user_email'] ) || $user !== false ) $this->error( '恐れ入りますが、最初からやり直してください' );

		update_option( 'user_tokens', $tokens );
	}
	// POSTアクション
	public function signupRegist() {

		// 入力値
		$inputs = array(
			'login_id',
			'nickname',
			'email',
			'password',
			'confirm_password',
		);
		foreach ( $inputs as $input ) {
			${$input} = ( isset( $_POST[$input] ) ? $_POST[$input] : '' );
		}

		// バリデーション
		if ( empty( $nickname ) ) $this->error( 'お名前を入力してください' );
		if ( empty( $login_id ) ) $this->error( 'ログインIDを入力してください' );
		if ( !preg_match("/^[a-zA-Z0-9_]+$/", $login_id) ) $this->error( 'ログインIDは英数字とアンダーバーのみ使用できます。' );
		if ( empty( $email )) $this->error( 'メールアドレスを入力してください' );
		if ( empty( $password ) ) $this->error( 'パスワードを入力してください' );
		if ( strlen($password) < 5 ) $this->error( 'パスワードは5文字以上入力してください' );
		if ( $confirm_password !== $password ) $this->error( 'パスワードが違います' );

		// 既存確認
		$user = get_userdatabylogin( $login_id );
		if ( $user !== false ) $this->error( 'そのログインIDは既に使用されています' );
		$user =  get_user_by( 'email', $email );
		if ( $user !== false ) $this->error( 'そのメールアドレスは既に使用されています' );

		// ユーザー作成
		$result = wp_insert_user( array(
			'user_login' => $login_id,
			'user_pass' => $password,
			'user_email' => $email,
			'nickname' => $nickname,
			'role' => 'author'
		) );

		// // 結果
		if ( !is_numeric( $result ) ) {
			wp_die( $result );
		}
	}

	///////////////////////////////////////
	/* エラー出力 */
	///////////////////////////////////////
	public function error($message) {
		wp_die($message, $message, array('back_link' => true));
	}
}
