<?php
new MairudoBunkoAdmin();

 class MairudoBunkoAdmin {

	public function __construct() {
		add_action( 'admin_init', array( $this, 'admin_init' ) );
		add_action( 'admin_menu', array( $this, 'admin_menu' ) );
	}

	///////////////////////////////////////
	/* 初期定義（管理） */
	///////////////////////////////////////
	public function admin_init() {

		// JavaScript登録
		add_action( 'admin_enqueue_scripts', array( $this, 'add_admin_scripts' ) );
		// CSS登録
		add_action( 'admin_print_styles', array( $this, 'add_admin_styles' ) );

		// 作業ボタン追加
		add_filter( 'page_row_actions', array( $this, 'add_novel_action' ), 10, 2 );

		// クイック編集のタクソノミー削除
		add_filter( 'quick_edit_show_taxonomy', array( $this, 'quick_edit_taxonomy_remove' ), 10, 3 );

		// 検索フィルター追加
		add_action( 'restrict_manage_posts', array( $this, 'add_search_filter' ) );
		add_filter( 'parse_query', array( $this, 'add_search_query' ) );
		add_filter( 'posts_where', array( $this, 'change_query_where' ) );

		// カスタムフィールド追加
		add_action( 'add_meta_boxes', array( $this, 'add_meta_box' ) );

		// 親記事指定
		add_filter( 'page_attributes_dropdown_pages_args', array( $this, 'custom_attributes_dropdown' ) );

		// 小説投稿時
		add_action( 'save_post', array( $this, 'save_post' ), 10, 3 );

		// テーマ有効化
		$this->theme_activated();
	}

	///////////////////////////////////////
	/* テーマ有効化 */
	///////////////////////////////////////
	public function theme_activated() {
		global $pagenow;
		if ( $pagenow == 'themes.php' && isset( $_GET['activated'] ) ) {
			// オプション初期化
			$options = get_option( 'mairudo_bunko_option', array() );
			update_option( 'mairudo_bunko_option', array_merge( unserialize( MAIRUDO_BUNKO_OPSIONS ), $options ) );
		}
	}

	///////////////////////////////////////
	/* 管理メニュー定義 */
	///////////////////////////////////////
	public function admin_menu() {
		global $pagenow;

		// 「投稿」「固定ページ」削除
		remove_menu_page( 'edit.php' );
		remove_menu_page( 'edit.php?post_type=page' );

		// 管理者以外はツール使用不可にする
		// ※Press This対応化時に外す
		if ( !current_user_can( 'administrator' ) ) {
			remove_menu_page( 'tools.php' );
		}

		// アイキャッチ画像対応化
		// if ( in_array($pagenow, array( 'post-new.php', 'post.php' ) ) ) {}
		$post_type = ( isset( $_GET['post_type'] ) ? $_GET['post_type'] : 
									( isset( $_GET['post'] ) ? get_post_type( $_GET['post'] ) : '' )
									);

		if ( $post_type == 'novel' ) {
			if ( ( isset($_GET['parent_id']) || ( isset( $_GET['post'] ) && get_post( $_GET['post'] )->post_parent != 0 ) ) ) {
				// カテゴリー・タグのメタボックス削除
				remove_meta_box( 'novel_catdiv', 'novel', 'side' );
				remove_meta_box( 'tagsdiv-novel_tag', 'novel', 'side' );
				// コメント不可
				remove_meta_box( 'commentstatusdiv', 'novel', 'normal' );
				remove_meta_box( 'commentsdiv', 'novel', 'normal' );
				// フォーム事前処理（小説ページ作成）
				add_action( 'edit_form_top', array( $this, 'eidt_sub_post' ) );
			} else {
				// アイキャッチ対応化
				add_theme_support( 'post-thumbnails' );
				// フォーム事前処理（小説作成）
				add_action( 'edit_form_top', array( $this, 'edit_parent_post' ) );
			}
		} elseif ( $post_type == 'attachment' ) {
			// メディアページはコメント不可にする
			remove_meta_box( 'commentstatusdiv', 'attachment', 'normal' );
			remove_meta_box( 'commentsdiv', 'attachment', 'normal' );
		}

		// 属性メタボックス削除
		// remove_meta_box( 'pageparentdiv', 'novel', 'side' );

	}

	///////////////////////////////////////
	/* JavaScript登録 */
	///////////////////////////////////////
	public function add_admin_scripts() {
		wp_enqueue_script( 'jquery' );
		wp_enqueue_media();
	}

	///////////////////////////////////////
	/* CSS登録 */
	///////////////////////////////////////
	public function add_admin_styles() {
		wp_enqueue_style( 'bunko_admin_style', get_template_directory_uri(). '/css/admin.css' );
	}

	///////////////////////////////////////
	/* 検索フィルター追加 */
	///////////////////////////////////////
	public function add_search_filter() {
		if ( isset( $_GET['post_type'] ) && $_GET['post_type'] === 'novel' ) {
			$novels = get_posts( 'numberposts=-1&post_type=novel&post_parent=0&order=asc&suppress_filters=0' );
			$novel_id = isset( $_GET['novel_id'] ) ? $_GET['novel_id'] : 0 ;
			?>
			<select name="novel_id">
				<option value="">すべての小説</option>
				<?php foreach ( $novels as $novel ) : ?>
					<option value="<?php echo $novel->ID; ?>"<?php selected( $novel_id, $novel->ID ); ?>><?php echo $novel->post_title; ?></option>
				<?php endforeach; ?>
			</select>
			<?php
		}
	}
	public function add_search_query( $query ) {
		global $pagenow;
		if ( $pagenow === 'edit.php' && isset( $_GET['novel_id'] ) && !empty( $_GET['novel_id'] ) ) {
			$novel_id = absint( $_GET['novel_id'] );
			$query->query_vars['post__in'] = array( $novel_id );
			$query->query_vars['post_parent__in'] = array( $novel_id );
		}
	}
	public function change_query_where( $where ) {
		global $wpdb;
		$where = preg_replace(
			"/{$wpdb->posts}\.ID IN \((\d+)\) AND {$wpdb->posts}\.post_parent/",
			"{$wpdb->posts}.ID IN ($1) OR {$wpdb->posts}.post_parent",
			$where
		);
		return $where;
	}

	///////////////////////////////////////
	/* カスタムフィールド追加 */
	///////////////////////////////////////
	public function add_meta_box() {
		if ( isset( $_GET['parent_id'] ) || isset( $_GET['post'] ) )
			add_meta_box( 'menu_order_box', '順序', array( $this, 'menu_order_box' ), 'novel', 'side' );

		if ( !isset( $_GET['parent_id'] ) )
			add_meta_box( 'dream_box', '名前変換', array( $this, 'dream_box' ), 'novel', 'advanced' );
	}

	///////////////////////////////////////
	/* 順序カスタムフィールド */
	///////////////////////////////////////
	public function menu_order_box() {
		global $post;
		printf(
			'<input type="hidden" name="parent_id" value="%s" />'.
			'<p><label class="screen-reader-text">順序</label><input type="text" name="menu_order" value="%s" size="4" /></p>',
			esc_attr( ( isset( $_GET['parent_id'] ) ? $_GET['parent_id'] : $post->post_parent ) ),
			esc_attr( $post->menu_order )
		);
	}

	///////////////////////////////////////
	/* 名前変換カスタムフィールド */
	///////////////////////////////////////
	public function dream_box() {
		global $post;
		$dreams = get_post_meta_arr( $post->ID, 'dreams', true );

		echo '<div class="dreams">';
		foreach ( $dreams as $idx => $dream ) {
			echo '<div class="dream">';
				echo '<label>名前' . ( $idx + 1 ) . '：<input type="text" name="dreams[' . $idx . '][name]" value="' . $dream['name'] . '" /></label> ';
				echo '<label>説明' . ( $idx + 1 ) . '：<input type="text" name="dreams[' . $idx . '][description]" value="' . $dream['description'] . '" /></label>';
			echo '</div>';
		}
		echo '</div>';
?>
<hr />
<button type="button" class="button" data-toggle="add-dream">追加</button>
<button type="button" class="button" data-toggle="remove-dream">削除</button>
<script type="text/javascript">
jQuery(document).ready(function($) {
	$('[data-toggle="add-dream"]').on('click', function() {
		var $dreamBox = $('#dream_box');
		var addNum = $dreamBox.find('.dream').length;
		var addDream =
		'<div class="dream">' +
			'<label>名前' + (addNum+1) + '：<input type="text" name="dreams[' + addNum + '][name]" value="" placeholder="例）太郎" /></label> ' +
			'<label>説明' + (addNum+1) + '：<input type="text" name="dreams[' + addNum + '][description]" value="" placeholder="例）主人公" /></label>' +
		'</div>';
		$dreamBox.find('.dreams').append(addDream);
	});
	$('[data-toggle="remove-dream"]').on('click', function() {
		$('#dream_box').find('.dream:last-child').remove();
	});
});
</script>
<?php
	}

	///////////////////////////////////////
	/* 作業ボタン追加 */
	///////////////////////////////////////
	public function add_novel_action( $actions, $post ) {
		if ( $post->post_type == 'novel' && $post->post_parent == 0 ) {
			$actions = array_merge( array(
				'add' => '<a href="'. get_admin_url( null, 'post-new.php?post_type=novel&parent_id='. $post->ID). '" title="追加">追加</a>'
			), $actions );
		}
		return $actions;
	}

	///////////////////////////////////////
	/* クイック編集のタクソノミー削除 */
	///////////////////////////////////////
	public function quick_edit_taxonomy_remove( $show_in_quick_edit, $taxonomy_name, $post_type ) {
		if ( $post_type === 'novel' )
			$show_in_quick_edit = false;
		return $show_in_quick_edit;
	}

	///////////////////////////////////////
	/* 小説ページ作成フォーム */
	///////////////////////////////////////
	public function eidt_sub_post() {
		$post_id = ( isset( $_GET['parent_id'] ) ? $_GET['parent_id'] : get_post( $_GET['post'] )->post_parent );
		echo '<p>この投稿は小説「<strong>'. get_the_title( $post_id ). '</strong>」のページとして扱われます。</p>';
	}

	///////////////////////////////////////
	/* 小説作成 */
	///////////////////////////////////////
	public function edit_parent_post() {
		global $post;
		if ( $post->post_type == 'novel' ) {
			echo '<p>※この投稿の本文は小説のあらすじとして使用されます。<br>';
			echo '小説のページを作成するには、投稿後に小説一覧から対象小説の「追加」リンクを選択してください。</p>';
		}
	}

	///////////////////////////////////////
	/* 親記事指定 */
	///////////////////////////////////////
	public function custom_attributes_dropdown( $dropdown_args ) {
		$dropdown_args['depth'] = 1;
		return $dropdown_args;
	}

	///////////////////////////////////////
	/* 小説投稿時 */
	///////////////////////////////////////
	public function save_post( $post_id, $post, $update ) {

		if ( ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) || $post->post_status === 'auto-draft' || wp_is_post_revision( $post_id ) ) 
			return $post_id;

		if ( $post->post_type == 'novel' && $post->post_parent == 0 ) {

			// 小説デフォルトカスタムフィールド
			$novel_votes = get_post_meta( $post_id, 'novel_votes', true );
			$novel_vote_count = get_post_meta( $post_id, 'novel_vote_count', true );
			$novel_views = get_post_meta( $post_id, 'novel_views', true );
			$novel_view_count = get_post_meta( $post_id, 'novel_view_count', true );
			$novel_favorite_count = get_post_meta( $post_id, 'novel_favorite_count', true );

			// 投票数初期化
			if ( $novel_votes === '' )
				add_post_meta( $post_id, 'novel_votes', array() );
			if ( $novel_vote_count === '' )
				add_post_meta( $post_id, 'novel_vote_count', 0 );
			// 閲覧数初期化
			if ( $novel_views === '' )
				add_post_meta( $post_id, 'novel_views', array() );
			if ( $novel_view_count === '' )
				add_post_meta( $post_id, 'novel_view_count', 0 );
			// お気に入り数初期化
			if ( $novel_favorite_count === '' )
				add_post_meta( $post_id, 'novel_favorite_count', 0 );

			// 名前変換
			if ( isset( $_POST['dreams'] ) ) {
				$dreams = array();
				foreach ( $_POST['dreams'] as $dream ) {
					$dreams[] = array(
						'name' => esc_attr( $dream['name'] ),
						'description' => esc_attr( $dream['description'] )
					);
				}
				update_post_meta( $post_id, 'dreams', $dreams );
			}
		}
	}

}
