<?php
new MairudoBunko();

 class MairudoBunko {

	public function __construct() {

		// 設定値定義
		global $mairudo_bunko_option;
		$mairudo_bunko_option = get_option( 'mairudo_bunko_option' );

		// ポスト投稿定義
		$this->add_post_type();

		if ( is_admin() ) {

			// Ajax
			if ( defined( 'DOING_AJAX' ) && DOING_AJAX ) {
				require_once get_template_directory(). '/inc/ajax.php';
			} else {
				require_once get_template_directory(). '/inc/admin.php';
				require_once get_template_directory(). '/inc/setting.php';
			}

		} else {

			add_action( 'init', array( $this, 'init' ) );

		}

		// ウィジェット
		require_once get_template_directory(). '/inc/widget.php';

		// クエリー変更
		add_action( 'pre_get_posts', array( $this, 'custom_posts_query' ) );
		add_action( 'pre_get_comments', array( $this, 'custom_comments_query' ) );

	}

	///////////////////////////////////////
	/* 初期定義 */
	///////////////////////////////////////
	public function init() {
		// 処理分岐
		require_once get_template_directory(). '/inc/action.php';

		// CSS&JavaScript登録
		add_action( 'wp_enqueue_scripts', array( $this, 'add_public_scripts' ) );

		// ログインページ
		add_action( 'login_head', array( $this, 'login_head' ) );

		// 検索タイトル変更
		add_filter( 'wp_title', array( $this, 'custom_search_title' ), 10, 2 );

		// 日付を確実に表示
		add_action( 'the_post', array( $this, 'show_date' ) );

		// 省略文字数定義
		add_filter( 'excerpt_length', array( $this, 'custom_excerpt_length' ) );

		// 名前変換
		add_filter( 'the_content', array( $this, 'dream' ) );
	}

	///////////////////////////////////////
	/* CSS&JavaScript登録 */
	///////////////////////////////////////
	public function add_public_scripts() {
		wp_enqueue_script( 'jquery' );
		wp_localize_script(
			'jquery',
			'_bunko',
			array(
				'ajax_url' => admin_url( 'admin-ajax.php' ),
				'ajax_nonce' => wp_create_nonce( 'bunko_ajax' )
			)
		);
	}

	///////////////////////////////////////
	/* ログインページ */
	///////////////////////////////////////
	public function login_head() {
		global $mairudo_bunko_option;
		if ( !empty( $mairudo_bunko_option['login_logo_image_id'] ) ) {
			list( $logo_url, $logo_w, $logo_h ) = wp_get_attachment_image_src( $mairudo_bunko_option['login_logo_image_id'], 'thumbnail' );
?>
<style type="text/css">
#login h1 a {
	width: <?php echo $logo_w; ?>px;
	height: <?php echo $logo_h; ?>px;
	background: url('<?php echo $logo_url;?>') no-repeat !important;
}
</style>
<?php
		}
	}

	///////////////////////////////////////
	/* 検索結果タイトル変更 */
	///////////////////////////////////////
	public function custom_search_title( $title, $sep ) {
		if ( is_search() ) {
			$title = '小説検索';
			if ( !empty( $sep ) )
				$title .= ' '. $sep. ' ';
		}
		return $title;
	}

	///////////////////////////////////////
	/* 投稿日付を確実に表示 */
	///////////////////////////////////////
	public function show_date() {
		global $previousday;
		$previousday = '';
	}

	///////////////////////////////////////
	/* 省略文字数定義 */
	///////////////////////////////////////
	public function custom_excerpt_length( $length ) {
		return MAIRUDO_BUNKO_EXCERPT_LENGTH;
	}

	///////////////////////////////////////
	/* 名前変換 */
	///////////////////////////////////////
	public function dream( $text ) {
		global $post;

		if ( $post->post_type == 'novel' ) {
			$post_id = ( $post->post_parent == 0 ) ? $post->ID : $post->post_parent;

			// 設定名前変換
			$dreams = get_post_meta_arr( $post_id, 'dreams', true );

			// ユーザー指定名前変換
			$cname = 'dream-' . $post_id;
			$dreamck = isset( $_COOKIE[$cname] ) ? $_COOKIE[$cname] : 'W10=';
			$us_dream = json_decode( base64_decode( $dreamck ), true );

			// 変換
			if ( !empty( $us_dream ) && !empty( $dreams ) ) {
				foreach ( $dreams as $idx => $dream ) {
					if ( isset( $us_dream[$idx]['name'] ) ) {
						$text = preg_replace('/' . $dream['name'] . '/', $us_dream[$idx]['name'], $text);
					}
				}
			}
		}

		return $text;
	}

	///////////////////////////////////////
	/* commentsクエリー変更 */
	///////////////////////////////////////
	public function custom_comments_query( $query ) {
		if ( is_admin() )
			$query->query_vars['post_author'] = get_current_user_id();
	}

	///////////////////////////////////////
	/* postsクエリー変更 */
	///////////////////////////////////////
	public function custom_posts_query( $query ) {

		if ( is_admin() ) {

			$query->set( 'author', get_current_user_id() );

		} else {

			if ( $query->is_main_query() ) {

				if ( isset( $query->query['post_type'] ) && $query->query['post_type'] == 'diary' ) {

					// 活動日記
					// アーカイブはID指定必須にする
					if ( $query->is_archive() && !isset( $_GET['uid'] ) ) {
						$query->is_404 = true;
					} else {
						$user = get_userdatabylogin( $_GET['uid'] ?? '' );
						if ( ! empty( $user ) ) {
							$query->set( 'author', $user->ID );
						}
					}

				} elseif ( is_user_logged_in() && isset( $_GET['act'] ) && $_GET['act'] == 'favorites' ) {

					// お気に入り小説
					$favorites = get_user_meta( get_current_user_id(), 'novel_favorites', true );
					if ( empty( $favorites ) ) $favorites = array(0);
					$query->set( 'post_type', array( 'novel' ) );
					$query->set( 'post_parent', 0 );
					$query->query_vars['post__in'] = $favorites;

				} elseif ( is_home() || $query->is_search ) {

					//  トップページ or 検索

					// 投稿タイプ
					$query->set( 'post_type', array( 'novel' ) );
					$query->set( 'post_parent', 0 );

					// タクソノミー
					$taxquery = array(
						'relation' => 'AND'
					);

					// カテゴリー
					if ( isset( $_GET['n_cat'] ) && is_array( $_GET['n_cat'] ) && !empty( $_GET['n_cat'] ) ) {
						$taxquery[] = array(
							'taxonomy' => 'novel_cat',
							'terms' => $_GET['n_cat'],
							'operator' => 'AND'
						);
					}

					// タグ
					if ( isset( $_GET['n_tag'] ) && is_array( $_GET['n_tag'] ) && !empty( $_GET['n_tag'] ) ) {
						$taxquery[] = array(
							'taxonomy' => 'novel_tag',
							'terms' => $_GET['n_tag'],
							'operator' => 'AND'
						);
					}

					// ソート
					if ( isset( $_GET['order'] ) ) {
						list( $orderby, $order, $meta ) = explode( ':', $_GET['order'] ?? '' );

						if ( !empty( $meta ) ) {
							$query->set( 'meta_key', $orderby );
							$query->set( 'orderby', 'meta_value_num' );
						} else {
							$query->set( 'orderby', $orderby );
						}

						$query->set( 'order', $order );
					}

					$query->set( 'tax_query', $taxquery );
				}
			}

		}

	}

	///////////////////////////////////////
	/* カスタム投稿定義 */
	///////////////////////////////////////
	public function add_post_type() {

		// 小説タイプ
		register_post_type(
			'novel',
			array(
				'label' => '小説一覧',
				'labels' => array(
					'name' => '小説',
					'singular_name' => '小説',
					'add_new_item' => '新規追加',
					'edit_item' => '編集',
					'new_item' => '新規小説',
					'all_items' => '小説一覧',
					'view_item' => '表示',
					'search_items' => '検索',
					'not_found' => '小説が見つかりませんでした。',
					'not_found_in_trash' => 'ゴミ箱内に小説が見つかりませんでした。',
				),
				'public' => true,
				'menu_icon' => 'dashicons-book',
				'hierarchical' => true,
				'has_archive' => true,
				'supports' => array(
					'title',
					'editor',
					'thumbnail',
					'comments',
					// 'page-attributes'
				),
				'menu_position' => 5
			)
		);
		// 小説カテゴリー
		register_taxonomy(
			'novel_cat',
			'novel',
			array(
				'label' =>  '小説カテゴリー',
				'labels' => array(
					'popular_items' => 'よく使う小説カテゴリー',
					'edit_item' => '小説カテゴリーを編集',
					'add_new_item' => '新規小説カテゴリーを追加',
					'search_items' =>  '小説カテゴリーを検索',
				),
				'public' => true,
				'hierarchical' => true,
				'rewrite' => array(
					'slug' => 'novel_cat',
					'with_front' => false,
					'feeds' => false,
					'pages' => true
				),
			)
		);
		// 小説タグ
		register_taxonomy(
			'novel_tag',
			'novel',
			array(
				'label' => '小説タグ',
				'labels' => array(
					'popular_items' =>  'よく使う小説タグ',
					'edit_item' =>'小説タグを編集',
					'add_new_item' => '新規小説タグを追加',
					'search_items' =>  '小説タグを検索',
				),
				'public' => true,
				'hierarchical' => false,
				'rewrite' => array(
					'slug' => 'novel_tag',
					'with_front' => false,
					'feeds' => false,
					'pages' => true
				),
			)
		);

		// 活動日記
		register_post_type(
			'diary',
			array(
				'label' => '活動日記',
				'labels' => array(
					'name' => '活動日記',
					'singular_name' => '活動日記',
					'add_new_item' => '新規追加',
					'edit_item' => '編集',
					'new_item' => '新規活動日記',
					'all_items' => '活動日記一覧',
					'view_item' => '表示',
					'search_items' => '検索',
					'not_found' => '活動日記が見つかりませんでした。',
					'not_found_in_trash' => 'ゴミ箱内に活動日記が見つかりませんでした。',
				),
				'public' => true,
				'menu_icon' => 'dashicons-calendar-alt',
				'has_archive' => true,
				'supports' => array(
					'title',
					'editor',
					'comments',
				),
				'menu_position' => 6
			)
		);

		//year
		add_rewrite_rule( 'diary/([0-9]{4})/?$', 'index.php?year=$matches[1]&post_type=diary', 'top' );
		add_rewrite_rule( 'diary/([0-9]{4})/page/?([0-9]{1,})/?$', 'index.php?year=$matches[1]&paged=$matches[2]&post_type=diary', 'top' );
		//month
		add_rewrite_rule( 'diary/([0-9]{4})/([0-9]{1,2})/?$', 'index.php?year=$matches[1]&monthnum=$matches[2]&post_type=diary', 'top' );
		add_rewrite_rule( 'diary/([0-9]{4})/([0-9]{1,2})/page/?([0-9]{1,})/?$', 'index.php?year=$matches[1]&monthnum=$matches[2]&paged=$matches[3]&post_type=diary', 'top' );
		//day
		add_rewrite_rule( 'diary/([0-9]{4})/([0-9]{1,2})/([0-9]{1,2})/?$', 'index.php?year=$matches[1]&monthnum=$matches[2]&day=$matches[3]&post_type=diary', 'top' );
		add_rewrite_rule( 'diary/([0-9]{4})/([0-9]{1,2})/([0-9]{1,2})/page/?([0-9]{1,})/?$', 'index.php?year=$matches[1]&monthnum=$matches[2]&day=$matches[3]&paged=$matches[4]&post_type=diary', 'top' );

		global $pagenow;
		if ( is_admin() && $pagenow == 'themes.php' && isset( $_GET['activated'] ) )
			flush_rewrite_rules();
	}

}
