<?php
new MairudoBunkoSetting();

class MairudoBunkoSetting {

	private $options;

	///////////////////////////////////////
	/* 初期定義 */
	///////////////////////////////////////
	public function __construct() {
		add_action( 'admin_menu', array( $this, 'add_page' ) );
		add_action( 'admin_init', array( $this, 'init' ) );
	}

	///////////////////////////////////////
	/* 設定ページ追加 */
	///////////////////////////////////////
	public function add_page() {
		add_theme_page(
			'テーマ設定', 
			'テーマ設定', 
			'manage_options', 
			'mairudo-bunko-setting', 
			array( $this, 'setting' )
		);
	}

	///////////////////////////////////////
	/* 設定ページ */
	///////////////////////////////////////
	public function setting() {
		?>
		<div class="wrap">
			<?php // screen_icon(); ?>
			<h2>テーマ設定</h2>
			<form method="post" action="options.php">
				<?php
				settings_fields( 'mairudo_bunko_setting' );   
				do_settings_sections( 'mairudo-bunko-setting' );
				submit_button(); 
				?>
			</form>
		</div>
		<?php
	}

	///////////////////////////////////////
	/* フィールド定義 */
	///////////////////////////////////////
	public function init() {
		global $mairudo_bunko_option;
		$this->options = $mairudo_bunko_option;

		register_setting(
			'mairudo_bunko_setting',
			'mairudo_bunko_option',
			array( $this, 'sanitize' )
		);

		/* * */

		add_settings_section(
			'mairudo_bunko_section_base',
			'基本設定',
			false, // array( $this, 'print_section_info' ),
			'mairudo-bunko-setting'
		);
			add_settings_field(
				'wrap_image_size',
				'画像サイズ',
				array( $this, 'wrap_image_size_callback' ),
				'mairudo-bunko-setting',
				'mairudo_bunko_section_base' 
			);
			add_settings_field(
				'create_account',
				'新規登録',
				array( $this, 'create_account_callback' ),
				'mairudo-bunko-setting',
				'mairudo_bunko_section_base' 
			);
			add_settings_field(
				'login_logo_image',
				'ログイン画面ロゴ画像',
				array( $this, 'login_logo_image_callback' ),
				'mairudo-bunko-setting',
				'mairudo_bunko_section_base' 
			);
	}

	///////////////////////////////////////
	/* サニタイズ */
	///////////////////////////////////////
	public function sanitize( $input ) {
		$new_input = array();

		$absint = array(
			'wrap_image_size',
			'create_account',
			'login_logo_image'
		);

		$sanitize_text_field = array(
		);

		foreach ( $input as $key => $value) {
			if (in_array($key, $absint)) {
				$value = absint($value);
			}
			if (in_array($key, $sanitize_text_field)) {
				$value = sanitize_text_field($value);
			}
			$new_input[$key] = $value;
		}

		return $new_input;
	}

	///////////////////////////////////////
	/* 画像サイズ */
	///////////////////////////////////////
	public function wrap_image_size_callback() {
		printf(
			'<label><input type="checkbox" name="mairudo_bunko_option[wrap_image_size]" value="1"%s />アップロード画像のサイズをテーマ指定にする</label>',
			checked( $this->options['wrap_image_size'], 1, false )
		);
	}

	///////////////////////////////////////
	/* 新規登録 */
	///////////////////////////////////////
	public function create_account_callback() {
		printf(
			'<label><input type="checkbox" name="mairudo_bunko_option[create_account]" value="1"%s />新規登録を許可する</label>',
			checked( $this->options['create_account'], 1, false )
		);
	}

	///////////////////////////////////////
	/* ログインページロゴ画像 */
	///////////////////////////////////////
	public function login_logo_image_callback() {
		if ( !empty( $this->options['login_logo_image_id'] ) ) {
			list( $logo_url, $logo_w, $logo_h ) = wp_get_attachment_image_src( $this->options['login_logo_image_id'], 'thumbnail' );
			$logo_img = '<img src="' . $logo_url . '" width="' . $logo_w . '" height="' . $logo_h . '" />';
		} else {
			$logo_img = '<img src="" />';
		}

?>
<script type="text/javascript">
jQuery(document).ready(function($) {
	var mb_media_frame;

	$(document).on('click', '#login_logo_image_add', function(e) {
		$this = $(this);

		if (mb_media_frame) {
			mb_media_frame.open();
			return;
		}

		mb_media_frame = wp.media.frames.mb_media_frame = wp.media({
			className: 'media-frame mb_media_frame',
			frame: 'post',
			multiple: false
		});

		mb_media_frame.on('insert', function() {
			var attachment = mb_media_frame.state().get('selection').first().toJSON();
			if ('image' != attachment.type || 'undefined' == typeof attachment.sizes) {
				alert('画像を選択してください');
				return;
			}
			file_url = attachment.sizes.full.url;
			if ('undefined' != typeof attachment.sizes.thumbnail) {
				file_url = attachment.sizes.thumbnail.url;
			}
			$this.hide();
			$this.siblings('#login_logo_image_del').show();
			$this.siblings('[name="mairudo_bunko_option[login_logo_image_id]"]').val(attachment.id);
			$this.siblings('#login_logo_image_view').children('img').prop('src', file_url);
		});

		mb_media_frame.open();
		mb_media_frame.content.mode('upload');
	}).
	on('click', '#login_logo_image_del', function() {
		$(this).siblings('#login_logo_image_view').children('img').prop('src', '');
		$(this).siblings('[name="mairudo_bunko_option[login_logo_image_id]"]').val('');
		$(this).siblings('#login_logo_image_add').show();
		$(this).hide();
	});
});
</script>
<input id="login_logo_image" type="hidden" name="mairudo_bunko_option[login_logo_image_id]" value="<?php echo $this->options['login_logo_image_id']; ?>" />
<input id="login_logo_image_add" type="button" value="画像指定" class="button"<?php if ( !empty( $this->options['login_logo_image_id'] ) ) echo ' style="display: none;"'; ?> /> 
<input id="login_logo_image_del" type="button" value="削除" class="button"<?php if ( empty( $this->options['login_logo_image_id'] ) ) echo ' style="display: none;"'; ?> />
<div id="login_logo_image_view"><?php echo $logo_img; ?></div>
<?php
	}
}
